<h1 class="big">nose</h1>
<h2 class="big">is nicer testing for python</h2>

<p class="big">nose extends unittest to make testing easier.</p>

{{ body }}

<h1>Documentation</h1>

<table class="contentstable">
    <tr>
        <td width="50%">
            <a class="biglink" href="{{ pathto("testing") }}">
                Testing with nose</a>
            <p>Find out how to write, find and run tests using nose.<br/>
                <a href="{{ pathto("testing") }}">More &gt;</a></p>
        </td>
        <td width="50%">
            <a class="biglink" href="{{ pathto("developing") }}">
                Developing with nose</a>
            <p>Find out how to write your own plugins, and about nose
                internals.<br/> <a href="{{ pathto("developing") }}">More &gt;</a></p>
        </td>
    </tr>
    <tr>
        <td>
            <a class="biglink" href="{{ pathto("news") }}">
                News</a>
            <p>What's new in this release?<br/>
                <a href="{{ pathto("news") }}">More &gt;</a></p>
        </td>
        <td>
            <a class="biglink" href="{{ pathto("further_reading") }}">
                Further reading</a>
            <p>Plugin recipes and usage examples, trivia and other
                uncategorizable items.<br/>
                <a href="{{ pathto("further_reading") }}">More &gt;</a></p>
        </td>
    </tr>
    <tr>
        <td>
            <span class="biglink">Indices and tables</span>
            <ul>
                <li><a href="{{ pathto("genindex") }}">
                    Complete index</a></li>
                <li><a href="{{ pathto("modindex") }}">
                    Index of documented modules</a></li>
            </ul>
        </td>
        <td></td>
    </tr>
</table>
