{% extends "!layout.html" %}

{%- block relbar1 %}
    {% if pagename != 'index' %}{{ super() }}{% endif %}
{% endblock %}

{%- block footer %}
    {{ super() }}
    {% if not embedded %}
        <script src="http://www.google-analytics.com/urchin.js"
                type="text/javascript">
        </script>
        <script type="text/javascript">
            _uacct = "UA-2236166-1";
            urchinTracker();
        </script>
    {% endif %}
{%- endblock %}
